function [ PR ROC ] = calcPR( salmap, groundtruth, comparisonType )
%[PR ROC] = calcPR (salmap, groundtruth [, comparisonType])
%   calculate the PR and ROC of given saliency map vs ground truth
% inputs:
%   salmap         = saliency map, will be normalized 0 to 1
%   groundtruth    = a binary map of object segmentation
%   comparisonType = the type of comparison as follows:
%                    1 = object tightly segmented vs original ground truth
%                    2 = bounding boxes of both salmap and ground truth
%                    3 = same as #1 but with filled holes of salmap when
%                        thresholding (default)
% outputs:
%   PR  = precision-recall curve, where:
%         precision = PR(1:2:end,:);
%         recall    = PR(2:2:end,:);
%         plot(recall,precision);
%   ROC = ROC score, where:
%         Tpp = ROC(1:2:end,:);
%         Fpp = ROC(2:2:end,:);
%         plot(Fpp,Tpp);

if ~islogical(groundtruth)
    gt_all = groundtruth > .5;
else
    gt_all = groundtruth;
end

if nargin < 3
    % Comparison (Groundtruth vs salmaps) type
    % 1 = object tight segmented
    % 2 = bounding boxes / edges of the ground truth
    % 3 = filling in holes
    comparisonType = 3;
end
if comparisonType == 2
    gt_all = makeBoxMap(gt_all);
end

thresholdParam = 0:0.05:1;

if size(salmap,3) > 1
    newsal = rgb2gray(salmap);
else
    newsal = salmap;
end
newsal = double(newsal);
newsal = imresize(newsal,size(groundtruth),'nearest');
% normalization
newsal = newsal - min(min(newsal));
newsal = newsal ./ max(max(newsal));


precision = zeros(1,length(thresholdParam));
recall    = zeros(1,length(thresholdParam));
tpp       = zeros(1,length(thresholdParam));
fpp       = zeros(1,length(thresholdParam));
for threshNo = 1:length(thresholdParam)
    thresh = thresholdParam(threshNo);
    salIdx = newsal >= thresh;
    if comparisonType == 2
        salIdx = makeBoxMap(salIdx);
%         subplot(1,2,1);imshow(salIdx)
%         subplot(1,2,2);imshow(gt_all)
%         pause(.3);
    elseif comparisonType == 3;
%         subplot(1,2,1);imshow(salIdx)
        salIdx = imfill(salIdx,'holes');
%         subplot(1,2,2);imshow(salIdx)
%         pause(.3);
    end
    tp = mean2(gt_all & salIdx);
    fp = mean2(~gt_all & salIdx);
    fn = mean2(gt_all & ~salIdx);
    tn = mean2(~gt_all & ~salIdx);
    
    precision(threshNo) = tp / (tp+fp);
    recall(threshNo) = tp / (tp+fn);
    
    tpp(threshNo) = tp / (tp+fn);
    fpp(threshNo) = fp / (fp+tn);
    
end

PR = [precision; recall];
ROC = [tpp; fpp];

end

