#ifndef __MATRIX_COMPUTATION_TEMPLATE_H__
#define __MATRIX_COMPUTATION_TEMPLATE_H__

/*=======================================
 * variables:
 *	input: vector, matrix
 *	mtx: matrix
 *	vec: vector
 *	scalar: scalar
 *
 * function:
 * 1. MCMax, MCMin
 *	usage:
 *		result = MCMax(input);
 *	description:
 *		result = max(input(:));
 *
 * 2. MCAPlusScalar
 *	usage:
 *		MCAPlusScalar(input, scalar);
 *	description:
 *		input = input + scalar;
 *
 * 3. MCSum
 *	usage:
 *		result = MCSum(input);
 *	description:
 *		result = sum(input(:));
 *
 * 4. MCVecProd
 *	usage:
 *		result = MCVecProd(vec1, vec2);
 *	description:
 *		result = vec1*vec2';
 *
 * 5. MCExp
 *	usage:
 *		MCExp(input);
 *	description:
 *		input = exp(input);
 *
 * 6. MCTranspose
 *	usage:
 *		MCTranspose(mtxIn, mtxOut);
 *	description:
 *		mtxOut = mtxIn';
 *
 * 7. MCAbs
 *	usage:
 *		MCAbs(input);
 *	description:
 *		input = abs(input);
 *
 * 8. MCSquare
 *	usage:
 *		MCSquare(input);
 *	description:
 *		input = input.^2;
 *
 * 9. MCL2Distance
 *	usage:
 *		result = MCL2Distance(vec1, vec2);
 *	description:
 *		result = |vec1-vec2|;
 *
 * 10. MCDimMin
 *	usage:
 *		result = MCDimMin(inMtx, outVec, nDim);
 *	description:
 *		result = min(mtx,[],dim);
 *
 * 11. MCMean
 *	usage:
 *		result = MCMean(input);
 *	description:
 *		result = mean(input(:));
 *=======================================*/

#include <math.h>
#include "1dmem_template_class.h"
#include "2dmem_template_class.h"

//================ MCMin ================
// result = min(input(:));
template<class T> inline T MCMin (T* pInput, int nEle){
	T minT;
	int i;

	minT = pInput[0];
	for(i=1; i<nEle; i++){
		if(pInput[i]<minT)
			minT = pInput[i];
	}
	return minT;
}

template<class T> inline T MCMin (TC1DMem<T>& input){
	return MCMin(input.GetPointer(), input.m_size);
}

template<class T> inline T MCMin (TC2DMem<T>& input){
	return MCMin(input.Get1DPointer(), input.m_row*input.m_col);
}
//~~~~~~~~~~~~ end of MCMin ~~~~~~~~~~~~~

//================ MCMax ================
// result = max(input(:));
template<class T> inline T MCMax (T* pInput, int nEle){
	T maxT;
	int i;

	maxT = pInput[0];
	for(i=1; i<nEle; i++){
		if(pInput[i]>maxT)
			maxT = pInput[i];
	}
	return maxT;
}

template<class T> inline T MCMax (TC1DMem<T>& input){
	return MCMax(input.GetPointer(), input.m_size);
}

template<class T> inline T MCMax (TC2DMem<T>& input){
	return MCMax(input.Get1DPointer(), input.m_row*input.m_col);
}
//~~~~~~~~~~~~ end of MCMax ~~~~~~~~~~~~~

//============ MCAPlusScalar ============
// input = input + scalar;
template<class T> inline void MCAPlusScalar (T* pInput, T scalar, int nEle){
	int i;

	for(i=0; i<nEle; i++){
		pInput[i] += scalar;
	}
}

template<class T> inline void MCAPlusScalar (TC1DMem<T>& input, T scalar){
	MCAPlusScalar(input.GetPointer(), scalar, input.m_size);
}

template<class T> inline void MCAPlusScalar (TC2DMem<T>& input, T scalar){
	MCAPlusScalar(input.Get1DPointer(), scalar, input.m_row*input.m_col);
}
//~~~~~~~~~ end of MCAPlusScalar ~~~~~~~~

//================ MCSum ================
// result = sum(input(:));
template<class T> inline T MCSum (T* pInput, int nEle){
	T sumT;
	int i;

	sumT = pInput[0];
	for(i=1; i<nEle; i++){
		sumT += pInput[i];
	}
	return sumT;
}

template<class T> inline T MCSum (TC1DMem<T>& input){
	return MCSum(input.GetPointer(), input.m_size);
}

template<class T> inline T MCSum (TC2DMem<T>& input){
	return MCSum(input.Get1DPointer(), input.m_row*input.m_col);
}
//~~~~~~~~~~~~~ end of MCSum ~~~~~~~~~~~~

//============== MCVecProd ==============
// result = vec1*vec2';
template<class T> inline T MCVecProd (T* pVec1, T* pVec2, int nEle){
	T prodT;
	int i;

	prodT = pVec1[0]*pVec2[0];
	for(i=1; i<nEle; i++){
		prodT += pVec1[i]*pVec2[i];
	}
	return prodT;
}
//~~~~~~~~~~~ end of MCVecProd ~~~~~~~~~~

//================ MCExp ================
// input = exp(input);
template<class T> inline void MCExp (T* pInput, int nEle){
	int i;

	for(i=0; i<nEle; i++){
		pInput[i] = exp(pInput[i]);
	}
}

template<class T> inline void MCExp (TC1DMem<T>& input){
	MCExp(input.GetPointer(), input.m_size);
}

template<class T> inline void MCExp (TC2DMem<T>& input){
	MCExp(input.Get1DPointer(), input.m_row*input.m_col);
}
//~~~~~~~~~~~~~ end of MCExp ~~~~~~~~~~~~

//============= MCTranspose =============
// output = input';
template<class T> inline void MCTranspose (TC2DMem<T>& mtxIn, TC2DMem<T>& mtxOut){
	int i, j;

	mtxOut.Alloc2DMem(mtxIn.m_col, mtxIn.m_row);
	for(i=0; i<mtxOut.m_row; i++){
		for(j=0; j<mtxOut.m_col; j++){
			mtxOut[i][j] = mtxIn[j][i];
		}
	}
}
//~~~~~~~~~~ end of MCTranspose ~~~~~~~~~

//================ MCAbs ================
// input = abs(input);
template<class T> inline void MCAbs (T* pInput, int nEle){
	int i;

	for(i=0; i<nEle; i++){
		if(pInput[i]<0)
			pInput[i] = -pInput[i];
	}
}

template<class T> inline void MCAbs (TC1DMem<T>& input){
	MCAbs(input.GetPointer(), input.m_size);
}

template<class T> inline void MCAbs (TC2DMem<T>& input){
	MCAbs(input.Get1DPointer(), input.m_row*input.m_col);
}
//~~~~~~~~~~~~~ end of MCAbs ~~~~~~~~~~~~

//============== MCSquare ===============
// input = input.^2;
template<class T> inline void MCSquare (T* pInput, int nEle){
	int i;

	for(i=0; i<nEle; i++){
		pInput[i] = pInput[i]*pInput[i];
	}
}

template<class T> inline void MCSquare (TC1DMem<T>& input){
	MCSquare(input.GetPointer(), input.m_size);
}

template<class T> inline void MCSquare (TC2DMem<T>& input){
	MCSquare(input.Get1DPointer(), input.m_row*input.m_col);
}
//~~~~~~~~~~~ end of MCSquare ~~~~~~~~~~~

//============= MCMulScalar =============
// input = input*scalar;
template<class T> inline void MCMulScalar (T* pInput, T scalar, int nEle){
	int i;

	for(i=0; i<nEle; i++){
		pInput[i] = pInput[i]*scalar;
	}
}

template<class T> inline void MCMulScalar (TC1DMem<T>& input, T scalar){
	MCMulScalar(input.GetPointer(), scalar, input.m_size);
}

template<class T> inline void MCMulScalar (TC2DMem<T>& input, T scalar){
	MCMulScalar(input.Get1DPointer(), scalar, input.m_row*input.m_col);
}
//~~~~~~~~~~ end of MCMulScalar ~~~~~~~~~

//============= MCL2Distance ============
//		result = |input1-input2|;
template<class T> inline T MCL2Distance (T* pInput1, T* pInput2, int nEle){
	int i;
	TC1DMem<T> tmp;
	tmp.Alloc1DMem(nEle);

	for(i=0; i<nEle; i++){
		tmp[i] = pInput1[i]-pInput2[i];
	}
	return sqrt(MCVecProd(tmp.GetPointer(), tmp.GetPointer(), nEle));
}

template<class T> inline T MCL2Distance (TC1DMem<T>& input1, TC1DMem<T>& input2){
	return MCL2Distance(input1.GetPointer(), input2.GetPointer(), input1.m_size);
}

template<class T> inline T MCL2Distance (TC2DMem<T>& input1, TC2DMem<T>& input2){
	return MCL2Distance(input1.Get1DPointer(), input2.Get1DPointer(), input1.m_row*input1.m_col);
}
//~~~~~~~~~ end of MCL2Distance ~~~~~~~~~

//=============== MCDimMin ==============
// result = min(mtx,[],dim);
template<class T> inline void MCDimMin (TC2DMem<T>& inMtx, TC1DMem<T>& outVec, int nDim){
	int i, j;
	T tmp;
	//nDim: 1-row, 2-col
	if(1==nDim){
		outVec.Alloc1DMem(inMtx.m_row);
		for(i=0; i<inMtx.m_row; i++){
			outVec[i] = MCMin(inMtx[i], inMtx.m_col);
		}
	}else{
		outVec.Alloc1DMem(inMtx.m_col);
		for(i=0; i<inMtx.m_col; i++){
			tmp = inMtx[0][i];
			for(j=1; j<inMtx.m_row; j++){
				if(inMtx[j][i]<tmp)
					tmp = inMtx[j][i];
			}
			outVec[i] = tmp;
		}
	}
}
//~~~~~~~~~~~ end of MCDimMin ~~~~~~~~~~~


//================ MCMean ================
// result = mean(input(:));
template<class T> inline T MCMean (T* pInput, int nEle){
	T meanT;
	int i;

	meanT = pInput[0];
	for(i=1; i<nEle; i++){
		meanT += pInput[i];
	}
	meanT /= (T)(nEle);
	return meanT;
}

template<class T> inline T MCMean (TC1DMem<T>& input){
	return MCMean(input.GetPointer(), input.m_size);
}

template<class T> inline T MCMean (TC2DMem<T>& input){
	return MCMean(input.Get1DPointer(), input.m_row*input.m_col);
}
//~~~~~~~~~~~~ end of MCMin ~~~~~~~~~~~~~
#endif