#include "mex.h"

#include <stdio.h>
#include <memory.h>
#include <blas.h>
#include "1dmem_template_class.h"
#include "2dmem_template_class.h"
#include "matrix_computation_template.h"

/* image segmentation geometric blur distance
 *
 * Usage:
 * [disTable] = L2_distance(mtx1, mtx2);
 *
 * Input:
 * mtx1 - mtx1[double] | TC2DMem<double>| data matrix 1
 *		MATLAB: [nDescLength*nData1] -> C: [nData1*nDescLength]
 * mtx2 - mtx2[double] | TC2DMem<double>| data matrix 2
 *		MATLAB: [nDescLength*nData2] -> C: [nData2*nDescLength]
 *
 * Output:
 * disTable - [double] | TC2DMem<double> | distance table
 *		[nData1*nData2]
 *
 */

/*
link with libmwblas.lib
void dgemm(char* transa, char* transb, int* m, int* n, int* k, double* alpha,
	double* A, int* lda, double* B, int* ldb, double beta, double* C, int* ldc);

C = alpha * op(A) * op(B) + beta * C,
where op(X) = X or op(X) = transpose(X),

transa - specifies op(A). If transa == 'N' or 'n', op(A) = A. If transa == 'T', 't', 'C', or 'c', op(A) = transpose(A).
transb - specifies op(B). If transb == 'N' or 'n', op(B) = B. If transb == 'T', 't', 'C', or 'c', op(B) = transpose(B).
m - number of rows of matrix op(A) and rows of matrix C; m must be at least zero.
n - number of columns of matrix op(B) and number of columns of C; n must be at least zero.
k - number of columns of matrix op(A) and number of rows of op(B); k must be at least zero.
alpha - single-precision scalar multiplier applied to op(A) * op(B).
A - single-precision array of dimensions (lda, k) if transa == 'N' or 'n', and of dimensions (lda, m) otherwise.
	If transa == 'N' or 'n' lda must be at least max(1, m), otherwise lda must be at least max(1, k).
lda - leading dimension of two-dimensional array used to store matrix A.
B - single-precision array of dimensions (ldb, n) if transb == 'N' or 'n', and of dimensions (ldb, k) otherwise.
	If transb == 'N' or 'n' ldb must be at least max (1, k), otherwise ldb must be at least max(1, n).
ldb - leading dimension of two-dimensional array used to store matrix B.
beta - single-precision scalar multiplier applied to C. If zero, C does not have to be a valid input
C - single-precision array of dimensions (ldc, n); ldc must be at least max(1, m).
ldc - leading dimension of two-dimensional array used to store matrix C.
*/

/*
void BLASCALL(dgetrs)(const char *TRANS, const int *N, const int *NRHS, 
		      const double* A, const int *LDA, const int* IPIV, double* B,
		      const int *LDB, int *INFO);
*/

void AtB (const mxArray *mtx1, const mxArray *mtx2, const mxArray *outMtx){
	double *A, *B, *C, one = 1.0, zero = 0.0;
	size_t m,n,p;
	char *cht = "T";
	char *chn = "N";

	A = mxGetPr(mtx1);
	B = mxGetPr(mtx2);
	m = mxGetM(mtx1);
	p = mxGetN(mtx1);
	n = mxGetN(mtx2);

	C = mxGetPr(outMtx);

	/* Pass all arguments to Fortran by reference */
	dgemm(cht, chn, (ptrdiff_t*)(&p), (ptrdiff_t*)(&n), (ptrdiff_t*)(&m), &one, A, (ptrdiff_t*)(&m), B, (ptrdiff_t*)(&m), &zero, C, (ptrdiff_t*)(&p));
}

void AB (const mxArray *mtx1, const mxArray *mtx2, const mxArray *outMtx){
	double *A, *B, *C, one = 1.0, zero = 0.0;
	size_t m,n,p;
	char *chn = "N";

	A = mxGetPr(mtx1);
	B = mxGetPr(mtx2);
	m = mxGetM(mtx1);
	p = mxGetN(mtx1);
	n = mxGetN(mtx2);

	C = mxGetPr(outMtx);

	/* Pass all arguments to Fortran by reference */
	dgemm(chn, chn, (ptrdiff_t*)(&m), (ptrdiff_t*)(&n), (ptrdiff_t*)(&p), &one, A, (ptrdiff_t*)(&m), B, (ptrdiff_t*)(&p), &zero, C, (ptrdiff_t*)(&m));
}

void L2_distance (TC2DMem<double>& mtx1, TC2DMem<double>& mtx2, TC2DMem<double>& disMtx){
/* L2 distance_AtB
 *
 * Input:
 * mtx1 - M*D
 * mtx2 - N*D
 *		D: feature length
 * disMtx - M*N
 *		AtB matrix
 *
 * Output:
 * disMtx - M*N
 *		L2 distance matrix
 */
	int m, n, d;
	int i, j, k;
	double *p1, *p2;
	double dTmp;
	TC1DMem<double> aa, bb;

	d = mtx1.m_col;
	m = mtx1.m_row;
	n = mtx2.m_row;

	aa.Alloc1DMem(m);
	bb.Alloc1DMem(n);
	for(i=0; i<m; i++){
		aa[i] = MCVecProd(mtx1[i], mtx1[i], d);
	}
	for(i=0; i<n; i++){
		bb[i] = MCVecProd(mtx2[i], mtx2[i], d);
	}

	for(i=0; i<m; i++){
		for(j=0; j<n; j++){
			dTmp = aa[i] + bb[j] - 2*disMtx[i][j];
			if(dTmp>0)
				disMtx[i][j] = sqrt(dTmp);
			else
				disMtx[i][j] = 0;
		}
	}
}

/* Function Declarations */
void CheckInput(int nrhs, const mxArray *const prhs[]){
/*
 * Input:
 * mtx1 - mtx1[double] | TC2DMem<double>| data matrix 1
 *		MATLAB: [nDescLength*nData1] -> C: [nData1*nDescLength]
 * mtx2 - mtx2[double] | TC2DMem<double>| data matrix 2
 *		MATLAB: [nDescLength*nData2] -> C: [nData2*nDescLength]
 */
	const mxArray* pCurData;

    if(nrhs!=2)
        mexErrMsgTxt("L2_distance requires 2 Inputs.");

	if(!mxIsClass(prhs[0], "double"))	//check mtx1
		mexErrMsgTxt("in[0] mtx1 must be 'double'.");

	if(!mxIsClass(prhs[1], "double"))	//check mtx2
		mexErrMsgTxt("in[1] mtx2 must be 'double'.");
}

void TransferInputData (int nrhs, const mxArray *const prhs[],
						TC2DMem<double>& mtx1,
						TC2DMem<double>& mtx2){
/*
 * Input:
 * mtx1 - mtx1[double] | TC2DMem<double>| data matrix 1
 *		MATLAB: [nDescLength*nData1] -> C: [nData1*nDescLength]
 * mtx2 - mtx2[double] | TC2DMem<double>| data matrix 2
 *		MATLAB: [nDescLength*nData2] -> C: [nData2*nDescLength]
 */
	size_t nData1, nData2;
	size_t nDescLength;
	int i;
	int nDescIdx, nDescEleIdx;
	const mxArray* pCurMtx;

	const mwSize* pDims;

	double* pDouble;

	//check input
	CheckInput(nrhs, prhs);

	//[0] mtx1
	pDims = mxGetDimensions(prhs[0]);
	nDescLength = pDims[0];
	nData1 = pDims[1];

	pDouble = (double*)mxGetData(prhs[0]);
	mtx1.AssignMemory((int)(nData1), (int)(nDescLength), pDouble);
	/*
	mtx1.m_row = nData1;
	mtx1.m_col = nDescLength;
	mtx1.m_ppMem = (double**)malloc(mtx1.m_row*sizeof(double*));
	for(i=0; i<mtx1.m_row; i++){
		mtx1.m_ppMem[i] = pDouble;
		pDouble += nDescLength;
	}
	*/

	//[1] mtx2
	nData2 = mxGetDimensions(prhs[1])[1];

	pDouble = (double*)mxGetData(prhs[1]);
	mtx2.AssignMemory((int)(nData2), (int)(nDescLength), pDouble);
	/*
	mtx2.m_row = nData2;
	mtx2.m_col = nDescLength;
	mtx2.m_ppMem = (double**)malloc(mtx2.m_row*sizeof(double*));
	for(i=0; i<mtx2.m_row; i++){
		mtx2.m_ppMem[i] = pDouble;
		pDouble += nDescLength;
	}
	*/
}

void L2_distance(const mxArray* pMtx1, const mxArray* pMtx2, mxArray** ppDisTable);
void L2_distance (const mxArray* pMtx1, const mxArray* pMtx2, mxArray** ppDisTable){
/*
 * Input:
 * mtx1 - mtx1[double] | TC2DMem<double>| data matrix 1
 *		MATLAB: [nDescLength*nData1] -> C: [nData1*nDescLength]
 * mtx2 - mtx2[double] | TC2DMem<double>| data matrix 2
 *		MATLAB: [nDescLength*nData2] -> C: [nData2*nDescLength]
 *
 * Output:
 * disTable - [double] | TC2DMem<double> | distance table
 *		[nData1*nData2]
 */
	TC2DMem<double> mtx1, mtx2;
	TC2DMem<double> disTable;

	size_t nData1, nData2;
	size_t nDescLength;

	double* pDouble;
	int i, j;

	nDescLength = mxGetM(pMtx1);	//Get number of rows in mxArray
	nData1 = mxGetN(pMtx1);	//Get number of columns in mxArray
	nData2 = mxGetN(pMtx2);	//Get number of columns in mxArray

	(*ppDisTable) = mxCreateDoubleMatrix(nData1, nData2, mxREAL);
	pDouble = mxGetPr(*ppDisTable);
	disTable.AssignMemory((int)(nData2), (int)(nData1), pDouble);
	AtB(pMtx1, pMtx2, *ppDisTable);

	pDouble = mxGetPr(pMtx1);
	mtx1.AssignMemory((int)(nData1), (int)(nDescLength), pDouble);

	pDouble = mxGetPr(pMtx2);
	mtx2.AssignMemory((int)(nData2), (int)(nDescLength), pDouble);

	L2_distance(mtx2, mtx1, disTable);
}

void mexFunction (int nlhs, mxArray *plhs[], int nrhs, const mxArray *const prhs[]){
	//check input
	CheckInput(nrhs, prhs);

	L2_distance(prhs[0], prhs[1], &(plhs[0]));
}
